ToraRange_RangeFinder = {}

ToraRange_RangeFinder.SendTime = 0
ToraRange_RangeFinder.Distances = {}

function ToraRange_RangeFinder:GetDistances()
	local distances = {}

	local function shouldCopyFromFirst(a, b)
		local localIndex = ToraRange_Util:GetLocalGroupId()

		-- Copy if A is local, B is not local, B->A does not exist
		return b ~= localIndex or self.Distances[b] == nil or self.Distances[b][a] == nil
	end

	for idA, t in pairs(self.Distances) do
		distances[idA] = {}

		for idB, dstAB in pairs(t) do
			if not distances[idB] then
				distances[idB] = {}
			end

			if shouldCopyFromFirst(idA, idB) then
				distances[idA][idB] = dstAB
				distances[idB][idA] = dstAB
			end
		end
	end

	return distances
end

function ToraRange_RangeFinder:OnRangeInformation(frame, event, prefix, msg, channel, sender)
	if prefix ~= "ToraRange" then
		return
	end

	-- Ignore messages we sent
	if strlower(sender) == strlower(ToraRange_Util:GetFullName("player")) then
		return
	end

	-- Get the group index of the sender
	local groupId = ToraRange_Util:GetPlayerGroupIndex(sender)
	if not groupId then
		print("Could not find group id for " .. sender)
		return
	end

	self.Distances[groupId] = {}

	-- Extract the distances from the string
	if msg ~= "" then
		local playerDistanceStrings = { strsplit(",", msg) }

		for i, distanceString in pairs(playerDistanceStrings) do
			if distanceString ~= "" then
				local otherName, dst = strsplit(":", distanceString)
				local otherId = ToraRange_Util:GetPlayerGroupIndex(otherName)
				self.Distances[groupId][otherId] = tonumber(dst)
			end
		end
	end
end

function ToraRange_RangeFinder:SendRangeInformation()
	local distanceString = ""
	local playerGroupId = ToraRange_Util:GetLocalGroupId()

	if not playerGroupId then
		print("Failed to get group id for local player")
		return
	end

	-- Append the distance from the local player for each player
	-- to the distance string and send it
	for i, dst in pairs(self.Distances[playerGroupId]) do
		local name = ToraRange_Util:GetFullNameById(i)
		if name then
			distanceString = distanceString .. name .. ":" .. tostring(dst) .. ","
		end
	end

	SendAddonMessage("ToraRange", distanceString, ToraRange_Util:GetGroupChannel())
end

function ToraRange_RangeFinder:UpdateLocalInformation()
	local playerGroupId = ToraRange_Util:GetLocalGroupId()

	if not playerGroupId then
		print("Failed to get group id for local player")
		return
	end

	self.Distances[playerGroupId] = {}

	-- Get the distance from the local player to each relevant player
	-- and update the distance table
	for i = 1, ToraRange_Util:GetGroupSize() do
		if ToraRange_Util:IsPlayerRelevant(i) and i ~= playerGroupId then
			local groupId = ToraRange_Util:MakeGroupId(i)

			-- Get Distance
			self.Distances[playerGroupId][i] = ToraRange_Util:GetDistanceToUnit(groupId)
		else
			self.Distances[playerGroupId][i] = nil
		end
	end
end

-- Called every frame
function ToraRange_RangeFinder:OnUpdate(frame, dt)
	if UnitInRaid("player") or UnitInParty("player") then
		-- Update the local distance information
		self:UpdateLocalInformation()

		-- Send an update once it is time
	    self.SendTime = self.SendTime + dt
	    if self.SendTime >= ToraRange_Options_SendPeriod then
	        self:SendRangeInformation()
	        self.SendTime = 0
	    end
	end
end

-- Called when the party changes in any way at all
function ToraRange_RangeFinder:OnGroupChanged()
	self.Distances = {}
end

ToraRange_RangeFinder.Frame = CreateFrame("frame")

-- Setup the update event
ToraRange_RangeFinder.Frame:SetScript("OnUpdate", ToraRange_Util:Bind(ToraRange_RangeFinder, ToraRange_RangeFinder.OnUpdate))

-- Setup the communication event
ToraRange_RangeFinder.Frame:RegisterEvent("CHAT_MSG_ADDON");
ToraRange_RangeFinder.Frame:SetScript("OnEvent", ToraRange_Util:Bind(ToraRange_RangeFinder, ToraRange_RangeFinder.OnRangeInformation));
RegisterAddonMessagePrefix("ToraRange")

-- Setup the party changed event
ToraRange_RangeFinder.Frame2 = CreateFrame("frame")
ToraRange_RangeFinder.Frame2:RegisterEvent("GROUP_ROSTER_UPDATE")
ToraRange_RangeFinder.Frame2:SetScript("OnEvent",  ToraRange_Util:Bind(ToraRange_RangeFinder, ToraRange_RangeFinder.OnGroupChanged))
