ToraRange = {}
ToraRange.Positions = {}
ToraRange.InitialPositionRange = 30

ToraRange.ColorInRange = {r=1, g=0, b=0, a=0.8}
ToraRange.ColorOutOfRange = {r=0, g=1, b=0, a=0.8}
ToraRange.ColorSelf = {r=0, g=0, b=1, a=0.8}

-- Called every frame
function ToraRange:OnUpdate(frame, dt)
	-- Only solve positions when visible and in a group
	if ToraRange_Options_Visible and (UnitInRaid("player") or UnitInParty("player")) then
		-- Update the positions
	    local distances = ToraRange_RangeFinder:GetDistances()
		local immovable = { [ToraRange_Util:GetLocalGroupId()] = true }

		self.Positions = ToraRange_Solver:Solve(self.Positions, distances, immovable)

		-- Check whether a player has any distance information available
		local function hasKnownDistances(i)
			for j, t in pairs(distances) do
				if t[i] then
					return true
				end
			end

			return false
		end

		-- Update the UI
		local localGroupId = ToraRange_Util:GetLocalGroupId()

		ToraRange_Radar:Reset()
		for i, pos in pairs(self.Positions) do
			-- Only draw players with available distance information
			if i == localGroupId or hasKnownDistances(i) then
				local color

				if i == localGroupId then
					color = self.ColorSelf
				else
					local dst = math.sqrt(pos.x*pos.x + pos.y*pos.y)
					color = dst > ToraRange_Options_RangeWarning and self.ColorOutOfRange or self.ColorInRange
				end

				-- Pass which icon the player has (can be nil)
				color.IconIndex = GetRaidTargetIndex(ToraRange_Util:MakeGroupId(i))

				ToraRange_Radar:Add(pos.x, pos.y, color)
			end
		end
	end
end

-- Called when the party changes in any way at all
function ToraRange:OnGroupChanged()
	self.Positions = {}
	ToraRange_Radar:Reset()

	if UnitInRaid("player") or UnitInParty("player") then
		-- Initialize the positions randomly
		local function randomPos()
			return {
				x = self.InitialPositionRange * (math.random() - 0.5),
				y = self.InitialPositionRange * (math.random() - 0.5),
			}
		end

		for i = 1, ToraRange_Util:GetGroupSize() do
			if ToraRange_Util:IsPlayerRelevant(i) then
				self.Positions[i] = i == ToraRange_Util:GetLocalGroupId() and { x = 0, y = 0 } or randomPos()
			end
		end

		if not UnitInRaid("player") then
			self.Positions[5] = { x = 0, y = 0 }
		end
	end
end

ToraRange.Frame = CreateFrame("frame")

-- Setup the update event
ToraRange.Frame:SetScript("OnUpdate", ToraRange_Util:Bind(ToraRange, ToraRange.OnUpdate))

-- Setup the party changed event
ToraRange.Frame:RegisterEvent("GROUP_ROSTER_UPDATE")
ToraRange.Frame:SetScript("OnEvent", ToraRange_Util:Bind(ToraRange, ToraRange.OnGroupChanged))

print("|cff00CCFFToraRange loaded|r")
